@@ -49,12 +49,16 @@ static void foobar_widget_class_init	(FoobarWidgetClass *klass);
 static void foobar_widget_instance_init (FoobarWidget *foo);
 static void foobar_widget_realize	(GtkWidget *w);
 static void foobar_widget_destroy	(GtkObject *o);
-static void foobar_widget_size_allocate	(GtkWidget *w,
-					 GtkAllocation *alloc);
+static void foobar_widget_size_allocate	(GtkWidget *toplevel,
+					 GtkAllocation *allocation);
+static void foobar_widget_size_request	(GtkWidget *toplevel,
+					 GtkRequisition *requisition);
 static gboolean foobar_leave_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
 static gboolean foobar_enter_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
+static gboolean foobar_widget_popup_panel_menu (FoobarWidget *foobar);
+
 static void append_task_menu (FoobarWidget *foo, GtkMenuShell *menu_bar);
 static void setup_task_menu (FoobarWidget *foo);
 
@@ -62,6 +66,11 @@ static GList *foobars = NULL;
 
 static GtkWindowClass *foobar_widget_parent_class = NULL;
 
+enum {
+	POPUP_PANEL_MENU_SIGNAL,
+	WIDGET_LAST_SIGNAL
+};
+
 GType
 foobar_widget_get_type (void)
 {
@@ -88,16 +97,24 @@ foobar_widget_get_type (void)
 	return object_type;
 }
 
+static guint foobar_widget_signals [WIDGET_LAST_SIGNAL] = { 0 };
+
 static void
 foobar_widget_class_init (FoobarWidgetClass *klass)
 {
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
+	GtkBindingSet  *binding_set;
+
+	binding_set = gtk_binding_set_by_class (klass);
+
+	klass->popup_panel_menu = foobar_widget_popup_panel_menu;
 
 	object_class->destroy = foobar_widget_destroy;
 
-	widget_class->realize = foobar_widget_realize;
-	widget_class->size_allocate = foobar_widget_size_allocate;
+	widget_class->realize            = foobar_widget_realize;
+	widget_class->size_allocate      = foobar_widget_size_allocate;
+	widget_class->size_request       = foobar_widget_size_request;
 	widget_class->enter_notify_event = foobar_enter_notify;
 	widget_class->leave_notify_event = foobar_leave_notify;
 
@@ -107,6 +124,20 @@ foobar_widget_class_init (FoobarWidgetClass *klass)
 			     "GtkMenuBar::internal-padding = 0\n"
 			     "}\n"
 			     "widget \"*.panel-foobar-menubar\" style \"panel-foobar-menubar-style\"");
+
+	foobar_widget_signals [POPUP_PANEL_MENU_SIGNAL] =
+		g_signal_new ("popup_panel_menu",
+			     G_TYPE_FROM_CLASS (object_class),
+			     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			     G_STRUCT_OFFSET (FoobarWidgetClass, popup_panel_menu),
+			     NULL,
+			     NULL,
+			     panel_marshal_BOOLEAN__VOID,
+			     G_TYPE_BOOLEAN,
+			     0);
+
+	gtk_binding_entry_add_signal (binding_set, GDK_F10, GDK_CONTROL_MASK,
+				     "popup_panel_menu", 0);
 }
 
 static gboolean
@@ -783,28 +814,52 @@ foobar_widget_destroy (GtkObject *o)
 }
 
 static void
-foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
+foobar_widget_size_allocate (GtkWidget     *toplevel,
+			     GtkAllocation *allocation)
 {
-	if (GTK_WIDGET_CLASS (foobar_widget_parent_class)->size_allocate)
-		GTK_WIDGET_CLASS (foobar_widget_parent_class)->size_allocate (w, alloc);
+	FoobarWidget *foo;
 
-	if (GTK_WIDGET_REALIZED (w)) {
-		FoobarWidget *foo = FOOBAR_WIDGET (w);
-		xstuff_set_pos_size (w->window,
-				     multiscreen_x (foo->screen),
-				     multiscreen_y (foo->screen),
-				     alloc->width,
-				     alloc->height);
+	GTK_WIDGET_CLASS (foobar_widget_parent_class)->size_allocate (toplevel, allocation);
 
-		g_slist_foreach (panel_list, queue_panel_resize, NULL);
-		basep_border_queue_recalc (foo->screen);
+	if (!GTK_WIDGET_REALIZED (toplevel))
+		return;
 
-		xstuff_set_wmspec_strut (w->window,
-					 0 /* left */,
-					 0 /* right */,
-					 alloc->height /* top */,
-					 0 /* bottom */);
-	}
+	foo = FOOBAR_WIDGET (toplevel);
+
+	g_slist_foreach (panel_list, queue_panel_resize, NULL);
+	basep_border_queue_recalc (foo->screen);
+
+	xstuff_set_wmspec_strut (toplevel->window, 0, 0, allocation->height, 0);
+}
+
+static void
+foobar_widget_size_request (GtkWidget      *toplevel,
+			    GtkRequisition *requisition)
+{
+	FoobarWidget *foo;
+	int           old_height;
+
+	foo = FOOBAR_WIDGET (toplevel);
+
+	old_height = toplevel->requisition.height;
+
+	GTK_WIDGET_CLASS (foobar_widget_parent_class)->size_request (toplevel, requisition);
+
+	if (!GTK_WIDGET_REALIZED (toplevel))
+		return;
+
+	requisition->width = multiscreen_width (foo->screen);
+
+	if (requisition->height != old_height)
+		gtk_window_resize (GTK_WINDOW (toplevel),
+				   requisition->width,
+				   requisition->height);
+
+	xstuff_set_pos_size (toplevel->window,
+			     multiscreen_x (foo->screen),
+			     multiscreen_y (foo->screen),
+			     requisition->width,
+			     requisition->height);
 }
 
 GtkWidget *
@@ -818,9 +873,7 @@ foobar_widget_new (const char *panel_id,
 	if (foobar_widget_exists (screen))
 		return NULL;
 
-	foo = g_object_new (FOOBAR_TYPE_WIDGET,
-			    "width_request", multiscreen_width (screen),
-			    NULL);
+	foo = g_object_new (FOOBAR_TYPE_WIDGET, NULL);
 
 	if (panel_id)
 		panel_widget_set_id (PANEL_WIDGET (foo->panel), panel_id);
@@ -990,3 +1043,14 @@ foobar_widget_redo_window(FoobarWidget *foo)
 
 	gtk_widget_map(widget);
 }
+
+static gboolean
+foobar_widget_popup_panel_menu (FoobarWidget *foobar)
+{
+ 	gboolean retval;
+
+	g_signal_emit_by_name (foobar->panel, "popup_menu", &retval);
+
+	return retval;
+}
+
