@@ -158,15 +158,36 @@ drag_data_received_cb (GtkWidget        *widget,
 }
 
 static void
-destroy_launcher (GtkWidget *widget, gpointer data)
+destroy_launcher (GtkWidget *widget,
+		  Launcher  *launcher)
 {
-	Launcher *launcher = data;
-	GtkWidget *prop_dialog = launcher->prop_dialog;
+	launcher_properties_destroy (launcher);
+}
 
+void
+launcher_properties_destroy (Launcher *launcher)
+{
+	GtkWidget *dialog;
+
+	dialog = launcher->prop_dialog;
 	launcher->prop_dialog = NULL;
 
-	if (prop_dialog != NULL)
-		gtk_widget_destroy (prop_dialog);
+	if (dialog)
+		gtk_widget_destroy (dialog);
+}
+
+static void
+icon_theme_changed_cb (GnomeIconLoader *icon_loader,
+		       Launcher        *launcher)
+{
+	const char *icon;
+
+	g_return_if_fail (launcher != NULL);
+
+	icon = gnome_desktop_item_get_string (
+			launcher->ditem, GNOME_DESKTOP_ITEM_ICON);
+
+	button_widget_set_pixmap (BUTTON_WIDGET (launcher->button), icon, -1);
 }
 
 static void
@@ -174,6 +195,8 @@ free_launcher (gpointer data)
 {
 	Launcher *launcher = data;
 
+	g_signal_handler_disconnect (panel_icon_loader, launcher->icon_changed_signal);
+
 	launchers_to_hoard = g_slist_remove (launchers_to_hoard, launcher);
 
 	gnome_desktop_item_unref (launcher->ditem);
@@ -433,7 +456,7 @@ setup_button (Launcher *launcher)
 			      str, NULL);
 
 	/* Setup accessible name */
-	add_atk_name_desc (launcher->button, str, NULL);
+	panel_set_atk_name_desc (launcher->button, str, NULL);
 
 	g_free (str);
 
@@ -445,6 +468,10 @@ setup_button (Launcher *launcher)
 					      GNOME_DESKTOP_ITEM_ICON);
 	button_widget_set_pixmap (BUTTON_WIDGET (launcher->button), icon, -1);
 
+	launcher->icon_changed_signal =
+			g_signal_connect (panel_icon_loader, "changed",
+					  G_CALLBACK (icon_theme_changed_cb), launcher);
+
 	/* Setup help */
 	docpath = gnome_desktop_item_get_string (launcher->ditem,
 						 "X-GNOME-DocPath");
@@ -668,6 +695,7 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	const char  *profile;
 	const char  *temp_key;
 	char        *launcher_location;
+	Launcher    *launcher;
 
 	g_return_if_fail (panel_widget != NULL);
 	g_return_if_fail (gconf_key != NULL);
@@ -683,7 +711,10 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 		return;
 	}
         
-	load_launcher_applet (launcher_location, panel_widget, position, TRUE, gconf_key);
+	launcher = load_launcher_applet (launcher_location, panel_widget, position, TRUE, gconf_key);
+	if (launcher != NULL &&
+	    strstr (launcher_location, PANEL_LAUNCHERS_PATH) == NULL)
+		launcher_hoard (launcher);
 
 	g_free (launcher_location);
 }
@@ -720,9 +751,9 @@ load_launcher_applet_full (const char       *params,
 		panel_applet_add_callback (launcher->info,
 					   "properties",
 					   GTK_STOCK_PROPERTIES,
-					   _("Properties"));
+					   _("_Properties"));
 
-	panel_applet_add_callback (launcher->info, "help", GTK_STOCK_HELP, _("Help"));
+	panel_applet_add_callback (launcher->info, "help", GTK_STOCK_HELP, _("_Help"));
 
 	/* setup button according to ditem */
 	setup_button (launcher);
@@ -963,6 +994,8 @@ launcher_get_unique_uri (void)
 			g_free (full);
 			return uri;
 		}
+		
+		g_free (full);
 	}
 
 	g_assert_not_reached ();
